#B.2 Harmonization R Script
# Method for reformatting taxon names in NRSA8_diatom to species case, 
# running biodata_check, and applying revised species lists with slash groups in C.1 name revisions 20180727.csv.
# Written by Ian Bishop and Sylvia Lee
# Updated 2018-09-07


#################################
###---SET WORKING DIRECTORY---###
setwd("")


#####################
###---LIBRARIES---###

library(plyr)
library(stringr)
library(reshape2)
library(qdapTools)
library(vegan)

#####################
###---FUNCTIONS---###
source("functions/all_functions.R")

##################################################################################################################
###---IMPORT AND ADJUST Biodata Taxon List (Current, Deprecated, and Retired Names; find at Biodata website)---###

biodata <- read.csv("read_only/20171031.1025.taxon.csv", fileEncoding="UTF-8-BOM")

# Subset to diatoms only
biodata_diatom <- biodata[biodata$AlgalGroup=="Diatom",]

# Subset to relevant columns (bench name, taxon status, and published name)
biodata_diatom_reduced <- biodata_diatom[,c("BenchTaxonName","TaxonConceptState","PublishedTaxonName")]

# Remove benchname duplicates
biodata_diatom_reduced <- biodata_diatom_reduced[!duplicated(biodata_diatom_reduced$BenchTaxonName),]

# Remove "NAWQA", "NRSA", and "ANS" provisional names from this list so they aren't rolled up to genus later
biodata_diatom_reduced <- biodata_diatom_reduced[!grepl(" NAWQA ", as.character(biodata_diatom_reduced$BenchTaxonName)),]
biodata_diatom_reduced <- biodata_diatom_reduced[!grepl(" ANS ", as.character(biodata_diatom_reduced$BenchTaxonName)),]
biodata_diatom_reduced <- biodata_diatom_reduced[!grepl(" NRSA ", as.character(biodata_diatom_reduced$BenchTaxonName)),]


##########################################################
###---IMPORT AND ADJUST NRSA8 DATASET TO WIDE FORMAT---###

# Import raw count data
NRSA8_diatom<-read.csv("read_only/NRSA8_diatom.csv")

# Create a new column $NRSA_TAXON_NAME_trunc, which applies species case and removes author names from strings in $NRSA_TAXON_NAME
NRSA8_diatom$NRSA_TAXON_NAME_trunc <- capitalize(as.character(NRSA8_diatom$NRSA_TAXON_NAME))
NRSA8_diatom$NRSA_TAXON_NAME_trunc <- sapply(NRSA8_diatom$NRSA_TAXON_NAME_trunc, truncAuthor)

# Create new column #NRSA_TAXON_NAME_rolled by rolling up $NRSA_TAXON_NAME_trunc
NRSA8_diatom$NRSA_TAXON_NAME_rolled <- ifelse(grepl(" cf.", as.character(NRSA8_diatom$NRSA_TAXON_NAME_trunc), fixed = TRUE), sub(" .*", "", as.character(NRSA8_diatom$NRSA_TAXON_NAME_trunc)), as.character(NRSA8_diatom$NRSA_TAXON_NAME_trunc))
NRSA8_diatom$NRSA_TAXON_NAME_rolled <- ifelse(grepl(" spp.", as.character(NRSA8_diatom$NRSA_TAXON_NAME_rolled), fixed = TRUE), sub(" .*", "", as.character(NRSA8_diatom$NRSA_TAXON_NAME_rolled)), as.character(NRSA8_diatom$NRSA_TAXON_NAME_rolled))
NRSA8_diatom$NRSA_TAXON_NAME_rolled <- ifelse(grepl("?", as.character(NRSA8_diatom$NRSA_TAXON_NAME_rolled), fixed = TRUE), sub(" .*", "", as.character(NRSA8_diatom$NRSA_TAXON_NAME_rolled)), as.character(NRSA8_diatom$NRSA_TAXON_NAME_rolled))
NRSA8_diatom$NRSA_TAXON_NAME_rolled <- ifelse(grepl(" aff.", as.character(NRSA8_diatom$NRSA_TAXON_NAME_rolled), fixed = TRUE), sub(" .*", "", as.character(NRSA8_diatom$NRSA_TAXON_NAME_rolled)), as.character(NRSA8_diatom$NRSA_TAXON_NAME_rolled))

# Does the same thing as the four above in order to elevate "sp." without elevating "sp. NAWQA EAM". uses gsub too, which is faster.
NRSA8_diatom$NRSA_TAXON_NAME_rolled <- gsub("(\\s+sp\\.$)", "", NRSA8_diatom$NRSA_TAXON_NAME_rolled)

# Capitalize NRSA and NAWQA and associated analyst initials
NRSA8_diatom$NRSA_TAXON_NAME_rolled <- gsub("sp. (.*)", "sp. \\U\\1", NRSA8_diatom$NRSA_TAXON_NAME_rolled, perl=TRUE)

# Create species by samples matrix of data (wide format) with species case names
# This is the original data before name adjustments are applied
N8_wide <- dcast(NRSA8_diatom, UID ~ NRSA_TAXON_NAME_rolled, value.var="NO_VALVES_CNTD", fun.aggregate=sum)

# Export wide formatted data
write.csv(N8_wide, "output/N8_wide.csv", row.names = FALSE)


########################################################
###---CORRECT NRSA8 WIDE DATA WITH biodata_check()---###

# Temporarily rename "UID" to "Sample"
names(N8_wide)[1] <- "Sample"
write.csv(N8_wide, "output/N8_wide.csv", row.names = FALSE)
N8_OTUs <- data.frame(colnames(N8_wide[,-1]))
write.csv(N8_OTUs, "output/N8_OTUs.csv", row.names=FALSE)

#run biodata_check, using N8_wide, the biodata csv, saving first object in list ($UpdatedDataset) as N8B_wide
N8B_wide <- biodata_check(data_filename="output/N8_wide", biodata_filename = "read_only/20171031.1025.taxon", data_type = "Dataframe")$UpdatedDataset

#the conversion table resulting from biodata_check
N8B_list <- biodata_check(list_filename="output/N8_OTUs", biodata_filename = "read_only/20171031.1025.taxon", data_type = "List")
write.csv(N8B_list, "output/N8B_list.csv", row.names=FALSE)

# Rename "Sample" to "UID"
names(N8B_wide)[1] <- "UID"

# Rename "Sample" to "UID"
names(N8_wide)[1] <- "UID"
write.csv(N8_wide, "output/N8_wide.csv", row.names = FALSE)

# Export to CSV biodata adjusted OTUs
N8B_OTUs <- data.frame(colnames(N8B_wide[,-1]))
names(N8B_OTUs)[1] <- "biodata adjusted OTUs"
write.csv(N8B_OTUs, "output/N8B_OTUs.csv", row.names=F)


###################################################################################
###---CREATE BIODATA ADJUSTED DF COPIES, ADJUST WITH nameTranslate(), EXPORT TO CSV---###

# From N8B_wide (BioData adjusted df), different name adjustments will be applied.
# Make a new object, which is a copy of N8B_wide to apply the different name adjustments
s_mixed1 <- N8B_wide
s_inter1 <- N8B_wide
s_inter2 <- N8B_wide
s_mixed2 <- N8B_wide
s_inter3 <- N8B_wide
s_inter4 <- N8B_wide
s_mixed3 <- N8B_wide



######################################################
###---IMPORT SPECIES NAME TRANSLATION DATAFRAMES---###

# The imported list is updated by adding columns of revised OTU names. This iterative process has the following steps:
# Mixed1 species list (column 2 in csv file) results from manual revision of names that have been through biodata_check.
# Mixed2 species list (column 5 in csv file) results from random forest analysis of mixed1 to detect OTUs with high analyst signals. 
# Mixed3 (column 8 in csv file)) result from additional random forest runs and adjustments to OTUs to resolve analyst signals.
splist_mixed1 <- importSpeciesList("read_only/C.1 name revisions 20180727.csv", 2) # = mixed1
splist_inter1 <- importSpeciesList("read_only/C.1 name revisions 20180727.csv", 3) # = inter1
splist_inter2 <- importSpeciesList("read_only/C.1 name revisions 20180727.csv", 4) # = inter2
splist_mixed2 <- importSpeciesList("read_only/C.1 name revisions 20180727.csv", 5) # = mixed2
splist_inter3 <- importSpeciesList("read_only/C.1 name revisions 20180727.csv", 6) # = inter3
splist_inter4 <- importSpeciesList("read_only/C.1 name revisions 20180727.csv", 7) # = inter4
splist_mixed3 <- importSpeciesList("read_only/C.1 name revisions 20180727.csv", 8) # = mixed3


# Apply nameTranslate using specified species lists and biodata adjusted dataframes
ss_mixed1 <-nameTranslate(s_mixed1, splist_mixed1)
ss_inter1 <-nameTranslate(s_inter1, splist_inter1)
ss_inter2 <-nameTranslate(s_inter2, splist_inter2)
ss_mixed2 <-nameTranslate(s_mixed2, splist_mixed2)
ss_inter3 <-nameTranslate(s_inter3, splist_inter3)
ss_inter4 <-nameTranslate(s_inter4, splist_inter4)
ss_mixed3 <-nameTranslate(s_mixed3, splist_mixed3)



# Drop "Drop" OTU
ss_mixed1 <- ss_mixed1[, -which(names(ss_mixed1) %in% "Drop")]
ss_inter1 <- ss_inter1[, -which(names(ss_inter1) %in% "Drop")]
ss_inter2 <- ss_inter2[, -which(names(ss_inter2) %in% "Drop")]
ss_mixed2 <- ss_mixed2[, -which(names(ss_mixed2) %in% "Drop")]
ss_inter3 <- ss_inter3[, -which(names(ss_inter3) %in% "Drop")]
ss_inter4 <- ss_inter4[, -which(names(ss_inter4) %in% "Drop")]
ss_mixed3 <- ss_mixed3[, -which(names(ss_mixed3) %in% "Drop")]


# Export ss_mixes
write.csv(ss_mixed1,"output/ss_mixed1.csv", row.names = FALSE)
write.csv(ss_inter1,"output/ss_inter1.csv", row.names = FALSE)
write.csv(ss_inter2,"output/ss_inter2.csv", row.names = FALSE)
write.csv(ss_mixed2,"output/ss_mixed2.csv", row.names = FALSE)
write.csv(ss_inter3,"output/ss_inter3.csv", row.names = FALSE)
write.csv(ss_inter4,"output/ss_inter4.csv", row.names = FALSE)
write.csv(ss_mixed3,"output/ss_mixed3.csv", row.names = FALSE)


# You now have the first adjusted dataset with mixed taxonomy
# Next step is to make unique code names for all OTUs
# Then select samples by UIDs with complete taxonomist and environmental information




#######################
###---GENUS NAMES---###

###recode OTU codes to genus names using the first character string of the OTU name.
ss_genus <- ss_mixed1
#convert zeros to NA
ss_genus[ss_genus == 0] <- NA
#convert ss_genus wide format to long format
ss_genus_melt <- melt(ss_genus, id.vars="UID", variable.name="TAXON", value.name="COUNT")
#remove rows in long format that do not have any count information (value in COUNT column is NA)
ss_genus_melt <- ss_genus_melt[complete.cases(ss_genus_melt[,3]),]
#convert TAXON column to genus (e.g. Achnanthes acares --> Achnanthes)
ss_genus_melt$TAXON <- gsub( " .*$", "", ss_genus_melt$TAXON)
#convert from long to wide format, summing up all the newly identical genus TAXON names
ss_genus_wide <- dcast(ss_genus_melt, UID ~ TAXON, fun=sum)




############################
###---making OTU codes---###

#extract OTU names from multiple datasets and get a list of unique names
mixed1names <- colnames(ss_mixed1[-1])
genusnames <- colnames(ss_genus_wide[-1])
inter1names <- colnames(ss_inter1[-1])
inter2names <- colnames(ss_inter2[-1])
mixed2names <- colnames(ss_mixed2[-1])
inter3names <- colnames(ss_inter3[-1])
inter4names <- colnames(ss_inter4[-1])
mixed3names <- colnames(ss_mixed3[-1])



#combine mixed list names into one unique list
fullnames <- unique(c(mixed1names, genusnames, 
                      inter1names, inter2names, mixed2names, 
                      inter3names, inter4names, mixed3names
                      ))
names_unique <- data.frame(fullnames)

#use make.cepnames in vegan package to make codes
names_unique["codes"] <- make.cepnames(names_unique$fullnames)
write.csv(names_unique, "output/names_unique.csv")

#use nameTranslate to convert column names in ss_mixed* dfs 
ss_mixed1_coded <- nameTranslate(ss_mixed1, names_unique)
ss_genus_coded <- nameTranslate(ss_genus_wide, names_unique)
ss_inter1_coded <- nameTranslate(ss_inter1, names_unique)
ss_inter2_coded <- nameTranslate(ss_inter2, names_unique)
ss_mixed2_coded <- nameTranslate(ss_mixed2, names_unique)
ss_inter3_coded <- nameTranslate(ss_inter3, names_unique)
ss_inter4_coded <- nameTranslate(ss_inter4, names_unique)
ss_mixed3_coded <- nameTranslate(ss_mixed3, names_unique)


#########################################################################################
###---SUBSET WIDE FORMAT DATAFRAMES TO ONLY UIDS WHERE TAXONOMIST METADATA IS KNOWN---###

# Subset of UIDs with complete taxonomist information
taxonomist <- read.csv("read_only/0913_taxonomist.csv", header=T)

# select UIDs with complete taxonomist information
ss_mixed1_comp <- subset(ss_mixed1_coded, UID %in% taxonomist$UID)
ss_genus_comp <- subset(ss_genus_coded, UID %in% taxonomist$UID)
ss_inter1_comp <- subset(ss_inter1_coded, UID %in% taxonomist$UID)
ss_inter2_comp <- subset(ss_inter2_coded, UID %in% taxonomist$UID)
ss_mixed2_comp <- subset(ss_mixed2_coded, UID %in% taxonomist$UID)
ss_inter3_comp <- subset(ss_inter3_coded, UID %in% taxonomist$UID)
ss_inter4_comp <- subset(ss_inter4_coded, UID %in% taxonomist$UID)
ss_mixed3_comp <- subset(ss_mixed3_coded, UID %in% taxonomist$UID)


# convert UID to integer, arrange by UID
ss_mixed1_comp$UID <- as.integer(ss_mixed1_comp$UID)
ss_mixed1_comp <- arrange(ss_mixed1_comp, UID)
ss_inter1_comp$UID <- as.integer(ss_inter1_comp$UID)
ss_inter1_comp <- arrange(ss_inter1_comp, UID)
ss_inter2_comp$UID <- as.integer(ss_inter2_comp$UID)
ss_inter2_comp <- arrange(ss_inter2_comp, UID)
ss_mixed2_comp$UID <- as.integer(ss_mixed2_comp$UID)
ss_mixed2_comp <- arrange(ss_mixed2_comp, UID)
ss_inter3_comp$UID <- as.integer(ss_inter3_comp$UID)
ss_inter3_comp <- arrange(ss_inter3_comp, UID)
ss_inter4_comp$UID <- as.integer(ss_inter4_comp$UID)
ss_inter4_comp <- arrange(ss_inter4_comp, UID)
ss_mixed3_comp$UID <- as.integer(ss_mixed3_comp$UID)
ss_mixed3_comp <- arrange(ss_mixed3_comp, UID)

ss_genus_comp$UID <- as.integer(ss_genus_comp$UID)
ss_genus_comp <- arrange(ss_genus_comp, UID)


#########################################
###---EXPORT WIDE FORMAT DATAFRAMES---###

#write ss_mixed*_comps to csv
write.csv(ss_mixed1_comp, "output/ss_mixed1_comp.csv", row.names=FALSE)
write.csv(ss_inter1_comp, "output/ss_inter1_comp.csv", row.names=FALSE)
write.csv(ss_inter2_comp, "output/ss_inter2_comp.csv", row.names=FALSE)
write.csv(ss_mixed2_comp, "output/ss_mixed2_comp.csv", row.names=FALSE)
write.csv(ss_inter3_comp, "output/ss_inter3_comp.csv", row.names=FALSE)
write.csv(ss_inter4_comp, "output/ss_inter4_comp.csv", row.names=FALSE)
write.csv(ss_mixed3_comp, "output/ss_mixed3_comp.csv", row.names=FALSE)
write.csv(ss_genus_comp, "output/ss_genus_comp.csv", row.names=FALSE)

# OUTPUT: You now have species by samples matrices with complete sample information
# Next step is to detect OTUs with high analyst signals with randomForest using rfmod function in B.3 Random Forest Script.R

