###Analysis of original and harmonized datasets

#Method for reshaping data frames and analysis with non-metric multidimensional scaling ordination plots, 
#exporting metafiles of ordination plots with samples represented as 
#points and analyst groups represented as ellipsoid hulls,
#analysis of analyst signal in revised datasets using analysis of similarity and PERMANOVA,
#analysis of variation explained by total phosphorus using PERMANOVA, 
#and making species response curves for total phosphorus.

#################################
###---SET WORKING DIRECTORY---###
setwd("")

######################
###---LIBRARIES---####
library(vegan)
library(goeveg)
library(mgcv)


#####################
###---FUNCTIONS---###
source("functions/all_functions.R")


ss_genus_processed <- ssProcess(ss_genus_comp, taxonomist) # = genus
ss_mixed1_processed <- ssProcess(ss_mixed1_comp, taxonomist) # = mixed1
ss_mixed2_processed <- ssProcess(ss_mixed2_comp, taxonomist) # = mixed2
ss_mixed3_processed <- ssProcess(ss_mixed3_comp, taxonomist) # = mixed3


#run NMDS
mds3d_genusnoshare <- metaMDS(ss_genus_processed[,-1:-2], distance="bray", k=3, try=100, trymax=1000, noshare=0.05, parallel=7)
mds3d_mixed1noshare <- metaMDS(ss_mixed1_processed[,-1:-2], distance="bray", k=3, try=100, trymax=1000, noshare=0.05, parallel=7)
mds3d_mixed2noshare <- metaMDS(ss_mixed2_processed[,-1:-2], distance="bray", k=3, try=100, trymax=1000, noshare=0.05, parallel=7)
mds3d_mixed3noshare <- metaMDS(ss_mixed3_processed[,-1:-2], distance="bray", k=3, try=100, trymax=1000, noshare=0.05, parallel=7)



#make ellipsoid hull plots and export as metafiles, obtain goodness of fit results
win.metafile("plots/hulls_mds3d_genusnoshare.wmf")
plotEhulls(mds3d_genusnoshare, ss_genus_processed)
dev.off()
win.metafile("plots/hulls_mds3d_mixed1noshare.wmf")
plotEhulls(mds3d_mixed1noshare, ss_mixed1_processed)
dev.off()
win.metafile("plots/hulls_mds3d_mixed2noshare.wmf")
plotEhulls(mds3d_mixed2noshare, ss_mixed2_processed)
dev.off()
win.metafile("plots/hulls_mds3d_mixed3noshare.wmf")
plotEhulls(mds3d_mixed3noshare, ss_mixed3_processed)
dev.off()

#make traditional nmds ordination plots of sites as points, visualize analyst groups as different colors
#export as metafiles
win.metafile("plots/points_mds3d_genusnoshare.wmf")
plotPoints(mds3d_genusnoshare, ss_genus_processed)
dev.off()
win.metafile("plots/points_mds3d_mixed1noshare.wmf")
plotPoints(mds3d_mixed1noshare, ss_mixed1_processed)
dev.off()
win.metafile("plots/points_mds3d_mixed2noshare.wmf")
plotPoints(mds3d_mixed2noshare, ss_mixed2_processed)
dev.off()
win.metafile("plots/points_mds3d_mixed3noshare.wmf")
plotPoints(mds3d_mixed3noshare, ss_mixed3_processed)
dev.off()

##Select OTUs with taxid as one of top 5 predictors from random forest
nobs(subset(dfmse_genus, pred1 == "taxid"))
nobs(subset(dfmse_mixed1, pred1 == "taxid"))
nobs(subset(dfmse_mixed2, pred1 == "taxid"))
nobs(subset(dfmse_mixed3, pred1 == "taxid"))

subset(dfmse_genus, pred1 == "taxid" | pred2 == "taxid" | pred3 == "taxid" | pred4 == "taxid" | pred5 == "taxid")
subset(dfmse_mixed1, pred1 == "taxid" | pred2 == "taxid" | pred3 == "taxid" | pred4 == "taxid" | pred5 == "taxid")
subset(dfmse_mixed2, pred1 == "taxid" | pred2 == "taxid" | pred3 == "taxid" | pred4 == "taxid" | pred5 == "taxid")
subset(dfmse_mixed3, pred1 == "taxid" | pred2 == "taxid" | pred3 == "taxid" | pred4 == "taxid" | pred5 == "taxid")


##ANOSIM with taxid as predictor
anosim_genus <- anosim(ss_genus_processed[,-1:-2], ss_genus_processed[,2], permutations = 999, distance = "bray", strata = NULL,
                    parallel = 7)
anosim_mixed1 <- anosim(ss_mixed1_processed[,-1:-2], ss_mixed1_processed[,2], permutations = 999, distance = "bray", strata = NULL,
                        parallel = 7)
anosim_mixed2 <- anosim(ss_mixed2_processed[,-1:-2], ss_mixed2_processed[,2], permutations = 999, distance = "bray", strata = NULL,
                       parallel = 7)
anosim_mixed3 <- anosim(ss_mixed3_processed[,-1:-2], ss_mixed3_processed[,2], permutations = 999, distance = "bray", strata = NULL,
                        parallel = 7)


##############
#number of OTUs with PTL as first predictor from random forest
nobs(subset(dfmse_genus, pred1 == "PTL"))
nobs(subset(dfmse_mixed1, pred1 == "PTL"))
nobs(subset(dfmse_mixed2, pred1 == "PTL"))
nobs(subset(dfmse_mixed3, pred1 == "PTL"))

##Select OTUs with PTL as one of top 5 predictors from random forest
##Make subset processed site x species matrix of PTL OTUs
##run adonis for PTL
PTL_genus <- subset(dfmse_genus, pred1 == "PTL" | pred2 == "PTL" | pred3 == "PTL" | pred4 == "PTL" | pred5 == "PTL")
PTLotus_genus <- c("UID", as.character(PTL_genus$tname))
PTLdf_genus <- ss_genus_comp[PTLotus_genus]
PTLdfno0_genus <- PTLdf_genus[!!rowSums(abs(PTLdf_genus[-c(1)])),]
PTLwq_genus <- merge(wq, PTLdfno0_genus, by="UID")
PTLadonis_genus <- adonis(decostand(PTLwq_genus[,-1:-13], method="total") ~ PTL, data=PTLwq_genus, permutations = 999, method = "bray", parallel=7)

PTL_mixed1 <- subset(dfmse_mixed1, pred1 == "PTL" | pred2 == "PTL" | pred3 == "PTL" | pred4 == "PTL" | pred5 == "PTL")
PTLotus_mixed1 <- c("UID", as.character(PTL_mixed1$tname))
PTLdf_mixed1 <- ss_mixed1_comp[PTLotus_mixed1]
PTLdfno0_mixed1 <- PTLdf_mixed1[!!rowSums(abs(PTLdf_mixed1[-c(1)])),]
PTLwq_mixed1 <- merge(wq, PTLdfno0_mixed1, by="UID")
PTLadonis_mixed1 <- adonis(decostand(PTLwq_mixed1[,-1:-13], method="total") ~ PTL, data=PTLwq_mixed1, permutations = 999, method = "bray", parallel=7)

PTL_mixed2 <- subset(dfmse_mixed2, pred1 == "PTL" | pred2 == "PTL" | pred3 == "PTL" | pred4 == "PTL" | pred5 == "PTL")
PTLotus_mixed2 <- c("UID", as.character(PTL_mixed2$tname))
PTLdf_mixed2 <- ss_mixed2_comp[PTLotus_mixed2]
PTLdfno0_mixed2 <- PTLdf_mixed2[!!rowSums(abs(PTLdf_mixed2[-c(1)])),]
PTLwq_mixed2 <- merge(wq, PTLdfno0_mixed2, by="UID")
PTLadonis_mixed2 <- adonis(decostand(PTLwq_mixed2[,-1:-13], method="total") ~ PTL, data=PTLwq_mixed2, permutations = 999, method = "bray", parallel=7)

PTL_mixed3 <- subset(dfmse_mixed3, pred1 == "PTL" | pred2 == "PTL" | pred3 == "PTL" | pred4 == "PTL" | pred5 == "PTL")
PTLotus_mixed3 <- c("UID", as.character(PTL_mixed3$tname))
PTLdf_mixed3 <- ss_mixed3_comp[PTLotus_mixed3]
PTLdfno0_mixed3 <- PTLdf_mixed3[!!rowSums(abs(PTLdf_mixed3[-c(1)])),]
PTLwq_mixed3 <- merge(wq, PTLdfno0_mixed3, by="UID")
PTLadonis_mixed3 <- adonis(decostand(PTLwq_mixed3[,-1:-13], method="total") ~ PTL, data=PTLwq_mixed3, permutations = 999, method = "bray", parallel=7)

###############
#Make species response curves of OTUs with TP as first predictor
#Import raw TP data
wq_raw <- read.csv("read_only/waterchem_rawfull.csv", header=T)
PTL_raw <- data.frame(wq_raw[c("UID", "PTL")])
PTL_raw[,"logPTL"] <- log(PTL_raw$PTL)
PTL_raw[,"log10PTL"] <- log10(PTL_raw$PTL)


#obtain names of OTUs with TP as first predictor
PTL_genuspred1 <- data.frame(c(subset(dfmse_genus, pred1 == "PTL")))
PTL_genuspred1names <- c("UID", as.character(PTL_genuspred1$tname))

PTL_mixed1pred1 <- data.frame(c(subset(dfmse_mixed1, pred1 == "PTL")))
PTL_mixed1pred1names <- c("UID", as.character(PTL_mixed1pred1$tname))

PTL_mixed2pred1 <- data.frame(c(subset(dfmse_mixed2, pred1 == "PTL")))
PTL_mixed2pred1names <- c("UID", as.character(PTL_mixed2pred1$tname))

PTL_mixed3pred1 <- data.frame(c(subset(dfmse_mixed3, pred1 == "PTL")))
PTL_mixed3pred1names <- c("UID", as.character(PTL_mixed3pred1$tname))

#merge TP data with each dataset and subset to columns selected
#puts OTU columns in order of mean abundance if several plots needed
PTL_rawmixed1 <- merge(PTL_raw, PTLdfno0_mixed1[PTL_mixed1pred1names], by="UID")
PTL_rawmixed1sp <- PTL_rawmixed1[,-1:-4]
PTL_rawmixed1sp <- PTL_rawmixed1sp[,order(colMeans(PTL_rawmixed1sp))]
PTL_rawmixed1ordered <- data.frame(cbind(PTL_rawmixed1[,1:4], PTL_rawmixed1sp))

PTL_rawgenus <- merge(PTL_raw, PTLdfno0_genus[PTL_genuspred1names], by="UID")

PTL_rawmixed2 <- merge(PTL_raw, PTLdfno0_mixed2[PTL_mixed2pred1names], by="UID")
PTL_rawmixed2sp <- PTL_rawmixed2[,-1:-4]
PTL_rawmixed2sp <- PTL_rawmixed2sp[,order(colMeans(PTL_rawmixed2sp))]
PTL_rawmixed2ordered <- data.frame(cbind(PTL_rawmixed2[,1:4], PTL_rawmixed2sp))

PTL_rawmixed3 <- merge(PTL_raw, PTLdfno0_mixed3[PTL_mixed3pred1names], by="UID")
PTL_rawmixed3sp <- PTL_rawmixed3[,-1:-4]
PTL_rawmixed3sp <- PTL_rawmixed3sp[,order(colMeans(PTL_rawmixed3sp))]
PTL_rawmixed3ordered <- data.frame(cbind(PTL_rawmixed3[,1:4], PTL_rawmixed3sp))



#generate species response curves with GAM
#export metafiles of multiplot figures
win.metafile(width=10, height=15, "plots/gam_m1.wmf")
par(mfrow=c(5,3))
specresponse_sl(PTL_rawmixed1ordered[5], PTL_rawmixed1ordered$log10PTL, model="gam", main=colnames(PTL_rawmixed1ordered[5]), xlab="TP (ug/L)", xlim=c(0,2), ylim=c(0,0.25), bw=TRUE)
specresponse_sl(PTL_rawmixed1ordered[6], PTL_rawmixed1ordered$log10PTL, model="gam", main=colnames(PTL_rawmixed1ordered[6]), xlab="TP (ug/L)", xlim=c(0,2), ylim=c(0,0.25), bw=TRUE)
specresponse_sl(PTL_rawmixed1ordered[7], PTL_rawmixed1ordered$log10PTL, model="gam", main=colnames(PTL_rawmixed1ordered[7]), xlab="TP (ug/L)", xlim=c(0,2), ylim=c(0,0.25), bw=TRUE)
specresponse_sl(PTL_rawmixed1ordered[8], PTL_rawmixed1ordered$log10PTL, model="gam", main=colnames(PTL_rawmixed1ordered[8]), xlab="TP (ug/L)", xlim=c(0,2), ylim=c(0,0.25), bw=TRUE)
specresponse_sl(PTL_rawmixed1ordered[9], PTL_rawmixed1ordered$log10PTL, model="gam", main=colnames(PTL_rawmixed1ordered[9]), xlab="TP (ug/L)", xlim=c(0,2), ylim=c(0,0.25), bw=TRUE)
specresponse_sl(PTL_rawmixed1ordered[10], PTL_rawmixed1ordered$log10PTL, model="gam", main=colnames(PTL_rawmixed1ordered[10]), xlab="TP (ug/L)", xlim=c(0,2), ylim=c(0,0.25), bw=TRUE)
specresponse_sl(PTL_rawmixed1ordered[11], PTL_rawmixed1ordered$log10PTL, model="gam", main=colnames(PTL_rawmixed1ordered[11]), xlab="TP (ug/L)", xlim=c(0,2), ylim=c(0,0.25), bw=TRUE)
specresponse_sl(PTL_rawmixed1ordered[12], PTL_rawmixed1ordered$log10PTL, model="gam", main=colnames(PTL_rawmixed1ordered[12]), xlab="TP (ug/L)", xlim=c(0,2), ylim=c(0,0.25), bw=TRUE)
specresponse_sl(PTL_rawmixed1ordered[13], PTL_rawmixed1ordered$log10PTL, model="gam", main=colnames(PTL_rawmixed1ordered[13]), xlab="TP (ug/L)", xlim=c(0,2), ylim=c(0,0.25), bw=TRUE)
specresponse_sl(PTL_rawmixed1ordered[14], PTL_rawmixed1ordered$log10PTL, model="gam", main=colnames(PTL_rawmixed1ordered[14]), xlab="TP (ug/L)", xlim=c(0,2), ylim=c(0,1), bw=TRUE)
specresponse_sl(PTL_rawmixed1ordered[15], PTL_rawmixed1ordered$log10PTL, model="gam", main=colnames(PTL_rawmixed1ordered[15]), xlab="TP (ug/L)", xlim=c(0,2), ylim=c(0,1), bw=TRUE)
dev.off()

win.metafile(width=6, height=8, "plots/gam_genus.wmf")
par(mfrow=c(3,2))
specresponse_sl(PTL_rawgenus[5], PTL_rawgenus$log10PTL, model="gam", main="Ulnaria", xlab="TP (ug/L)", xlim=c(0,2), ylim=c(0,1), bw=TRUE)
specresponse_sl(PTL_rawgenus[6], PTL_rawgenus$log10PTL, model="gam", main="Encyonopsis", xlab="TP (ug/L)", xlim=c(0,2), ylim=c(0,1), bw=TRUE)
specresponse_sl(PTL_rawgenus[7], PTL_rawgenus$log10PTL, model="gam", main="Diadesmis", xlab="TP (ug/L)", xlim=c(0,2), ylim=c(0,0.25), bw=TRUE)
specresponse_sl(PTL_rawgenus[8], PTL_rawgenus$log10PTL, model="gam", main="Cymbella", xlab="TP (ug/L)", xlim=c(0,2), ylim=c(0,1), bw=TRUE)
specresponse_sl(PTL_rawgenus[9], PTL_rawgenus$log10PTL, model="gam", main="Gomphosphenia", xlab="TP (ug/L)", xlim=c(0,2), ylim=c(0,0.5), bw=TRUE)
dev.off()

win.metafile(width=10, height=15, "plots/gam_m2.wmf")
par(mfrow=c(5,3))
specresponse_sl(PTL_rawmixed2ordered[5], PTL_rawmixed2ordered$log10PTL, model="gam", main=colnames(PTL_rawmixed2ordered[5]), xlab="TP (ug/L)", xlim=c(0,2), ylim=c(0,0.25), bw=TRUE)
specresponse_sl(PTL_rawmixed2ordered[6], PTL_rawmixed2ordered$log10PTL, model="gam", main=colnames(PTL_rawmixed2ordered[6]), xlab="TP (ug/L)", xlim=c(0,2), ylim=c(0,0.25), bw=TRUE)
specresponse_sl(PTL_rawmixed2ordered[7], PTL_rawmixed2ordered$log10PTL, model="gam", main=colnames(PTL_rawmixed2ordered[7]), xlab="TP (ug/L)", xlim=c(0,2), ylim=c(0,0.25), bw=TRUE)
specresponse_sl(PTL_rawmixed2ordered[8], PTL_rawmixed2ordered$log10PTL, model="gam", main=colnames(PTL_rawmixed2ordered[8]), xlab="TP (ug/L)", xlim=c(0,2), ylim=c(0,0.25), bw=TRUE)
specresponse_sl(PTL_rawmixed2ordered[9], PTL_rawmixed2ordered$log10PTL, model="gam", main=colnames(PTL_rawmixed2ordered[9]), xlab="TP (ug/L)", xlim=c(0,2), ylim=c(0,0.25), bw=TRUE)
specresponse_sl(PTL_rawmixed2ordered[10], PTL_rawmixed2ordered$log10PTL, model="gam", main=colnames(PTL_rawmixed2ordered[10]), xlab="TP (ug/L)", xlim=c(0,2), ylim=c(0,0.5), bw=TRUE)
specresponse_sl(PTL_rawmixed2ordered[11], PTL_rawmixed2ordered$log10PTL, model="gam", main=colnames(PTL_rawmixed2ordered[11]), xlab="TP (ug/L)", xlim=c(0,2), ylim=c(0,0.25), bw=TRUE)
specresponse_sl(PTL_rawmixed2ordered[12], PTL_rawmixed2ordered$log10PTL, model="gam", main=colnames(PTL_rawmixed2ordered[12]), xlab="TP (ug/L)", xlim=c(0,2), ylim=c(0,0.25), bw=TRUE)
specresponse_sl(PTL_rawmixed2ordered[13], PTL_rawmixed2ordered$log10PTL, model="gam", main=colnames(PTL_rawmixed2ordered[13]), xlab="TP (ug/L)", xlim=c(0,2), ylim=c(0,0.25), bw=TRUE)
specresponse_sl(PTL_rawmixed2ordered[14], PTL_rawmixed2ordered$log10PTL, model="gam", main=colnames(PTL_rawmixed2ordered[14]), xlab="TP (ug/L)", xlim=c(0,2), ylim=c(0,0.5), bw=TRUE)
specresponse_sl(PTL_rawmixed2ordered[15], PTL_rawmixed2ordered$log10PTL, model="gam", main=colnames(PTL_rawmixed2ordered[15]), xlab="TP (ug/L)", xlim=c(0,2), ylim=c(0,1), bw=TRUE)
dev.off()

win.metafile(width=10, height=15, "plots/gam_m3.wmf")
par(mfrow=c(5,3))
specresponse_sl(PTL_rawmixed3ordered[5], PTL_rawmixed3ordered$log10PTL, model="gam", main=colnames(PTL_rawmixed3ordered[5]), xlab="TP (ug/L)", xlim=c(0,2), ylim=c(0,.25), bw=TRUE)
specresponse_sl(PTL_rawmixed3ordered[6], PTL_rawmixed3ordered$log10PTL, model="gam", main=colnames(PTL_rawmixed3ordered[6]), xlab="TP (ug/L)", xlim=c(0,2), ylim=c(0,.25), bw=TRUE)
specresponse_sl(PTL_rawmixed3ordered[7], PTL_rawmixed3ordered$log10PTL, model="gam", main=colnames(PTL_rawmixed3ordered[7]), xlab="TP (ug/L)", xlim=c(0,2), ylim=c(0,.25), bw=TRUE)
specresponse_sl(PTL_rawmixed3ordered[8], PTL_rawmixed3ordered$log10PTL, model="gam", main=colnames(PTL_rawmixed3ordered[8]), xlab="TP (ug/L)", xlim=c(0,2), ylim=c(0,.25), bw=TRUE)
specresponse_sl(PTL_rawmixed3ordered[9], PTL_rawmixed3ordered$log10PTL, model="gam", main=colnames(PTL_rawmixed3ordered[9]), xlab="TP (ug/L)", xlim=c(0,2), ylim=c(0,.25), bw=TRUE)
specresponse_sl(PTL_rawmixed3ordered[10], PTL_rawmixed3ordered$log10PTL, model="gam", main=colnames(PTL_rawmixed3ordered[10]), xlab="TP (ug/L)", xlim=c(0,2), ylim=c(0,.5), bw=TRUE)
specresponse_sl(PTL_rawmixed3ordered[11], PTL_rawmixed3ordered$log10PTL, model="gam", main=colnames(PTL_rawmixed3ordered[11]), xlab="TP (ug/L)", xlim=c(0,2), ylim=c(0,.25), bw=TRUE)
specresponse_sl(PTL_rawmixed3ordered[12], PTL_rawmixed3ordered$log10PTL, model="gam", main=colnames(PTL_rawmixed3ordered[12]), xlab="TP (ug/L)", xlim=c(0,2), ylim=c(0,.25), bw=TRUE)
specresponse_sl(PTL_rawmixed3ordered[13], PTL_rawmixed3ordered$log10PTL, model="gam", main=colnames(PTL_rawmixed3ordered[13]), xlab="TP (ug/L)", xlim=c(0,2), ylim=c(0,.25), bw=TRUE)
specresponse_sl(PTL_rawmixed3ordered[14], PTL_rawmixed3ordered$log10PTL, model="gam", main=colnames(PTL_rawmixed3ordered[14]), xlab="TP (ug/L)", xlim=c(0,2), ylim=c(0,1), bw=TRUE)
specresponse_sl(PTL_rawmixed3ordered[15], PTL_rawmixed3ordered$log10PTL, model="gam", main=colnames(PTL_rawmixed3ordered[15]), xlab="TP (ug/L)", xlim=c(0,2), ylim=c(0,1), bw=TRUE)
dev.off()