#FUNCTIONS

# Species case function
capitalize <- function(x){
  first <- toupper(substr(x, start=1, stop=1)) ## capitalize first letter
  rest <- tolower(substr(x, start=2, stop=nchar(x)))   ## everything else lowercase
  paste0(first, rest)
}

# This function removes all text after the specific epithet or variety or form, etc.
truncAuthor <- function(x){
  first  <- str_split_fixed(x, " ", n=6)[1] #extract first word
  second <- str_split_fixed(x, " ", n=6)[2] #extract second word
  third  <- str_split_fixed(x, " ", n=6)[3] #extract third word
  fourth <- str_split_fixed(x, " ", n=6)[4] #extract fourth word
  fifth  <- str_split_fixed(x, " ", n=6)[5] #extract fifth word
  
  if (grepl(" var. ", x) == TRUE) {
    paste(first, second, third, fourth)
  }
  else if (grepl(" fo. ", x) == TRUE) {
    paste(first, second, third, fourth)
  }
  else if (grepl(" sp. . .", x) == TRUE) {
    paste(first, second, third, fourth, fifth)
  }
  else {paste(first, second)}
}


#Function title: biodata_check()
#Author: Ian Bishop
#Date: 14 June 2017

biodata_check <- function(list_filename="", 
                          data_filename="", 
                          biodata_filename="", 
                          data_type=c("List", "Dataframe")) {
  
  #Libraries
  library(qdapTools)
  library(reshape2)
  
  #Import biodata complete taxonomy  
  biodata_names <- read.csv(paste(biodata_filename, ".csv", sep=""), header=TRUE, stringsAsFactors = FALSE, fileEncoding="UTF-8-BOM")
  
  
  if (data_type=="List") {
    
    #Import user list of taxa
    user_list <- read.csv(paste(list_filename, ".csv", sep=""), header=TRUE, stringsAsFactors = FALSE)
    
    #Add user list to new conversion dataframe
    conversion_df <- user_list
    names(conversion_df)[1] <- "UserTaxa"
    
    #Compare user names to Biodata bench names, 
    conversion_df$CurrentBiodataName <- lookup_e(conversion_df[,1], biodata_names[,c("BenchTaxonName", "BiodataTaxonName")])
    
    #Add column for change notes
    conversion_df$ConversionNotes <- NA
    conversion_df[is.na(conversion_df$CurrentBiodataName)==FALSE,]$ConversionNotes <- "Name Updated to Current Biodata Name"
    conversion_df[is.na(conversion_df$CurrentBiodataName)==TRUE,]$ConversionNotes <- "Name Not Found In Biodata"
    conversion_df$ConversionNotes[as.character(conversion_df$CurrentBiodataName)==as.character(conversion_df$UserTaxa)] <- "Name Already Current"
    
    #Replace NAs in CurrentBiodataName column with original user name.
    conversion_df[is.na(conversion_df$CurrentBiodataName)==TRUE,]$CurrentBiodataName <- conversion_df[is.na(conversion_df$CurrentBiodataName)==TRUE,]$UserTaxa
    
    #Convert conversion_df to dataframe.
    conversion_df <- data.frame(lapply(conversion_df, as.character), stringsAsFactors=FALSE)
    
    #Export conversion_df to .csv
    # write.csv(conversion_df, "OUTPUT_converted_names.csv", row.names = FALSE)
    
    return(conversion_df)
    
  } else if(data_type=="Dataframe") {
    
    #Import user wide format tidy dataset  
    user_data <- read.csv(paste(data_filename, ".csv", sep=""), check.names = FALSE, stringsAsFactors=FALSE, header=TRUE, fileEncoding="UTF-8-BOM")
    
    #Create local variable for imported data
    working_wide <- user_data
    
    #Convert working_wide to long format
    working_wide <- melt(working_wide, value.name="Count", variable.name="Taxon", id.vars="Sample")
    
    #Add BiodataTaxon column of current Biodata names
    working_wide$BiodataTaxon <- lookup_e(working_wide$Taxon, biodata_names[,c("BenchTaxonName", "BiodataTaxonName")])
    
    
    
    #Name translation sheet
    working_translation <- working_wide[,c("Taxon", "BiodataTaxon")]
    working_translation <- unique(working_translation)
    
    #Add column for change notes
    working_translation$ConversionNotes <- NA
    working_translation[is.na(working_translation$BiodataTaxon)==FALSE,]$ConversionNotes <- "Name Updated to Current Biodata Name"
    working_translation[is.na(working_translation$BiodataTaxon)==TRUE,]$ConversionNotes <- "Name Not Found In Biodata"
    working_translation$ConversionNotes[as.character(working_translation$BiodataTaxon)==as.character(working_translation$Taxon)] <- "Name Already Current"
    
    #Fill NAs in working_translation$BiodataTaxon with Taxon
    working_translation$BiodataTaxon[is.na(working_translation$BiodataTaxon)] <- as.character(working_translation$Taxon[is.na(working_translation$BiodataTaxon)])
    
    
    
    #Fill in BiodataTaxon NAs with original taxon.
    working_wide$BiodataTaxon[is.na(working_wide$BiodataTaxon)] <- as.character(working_wide$Taxon[is.na(working_wide$BiodataTaxon)])
    
    #Collected list of names that have been changed
    changed_names <- as.data.frame(unique(working_wide[working_wide$Taxon != working_wide$BiodataTaxon,]$Taxon))
    colnames(changed_names) <- "Changed Names"
    
    
    
    #Replace NAs in BiodataTaxon column with original Taxon.
    working_wide$Taxon <- working_wide$BiodataTaxon
    #Remove BiodataTaxon column
    working_wide$BiodataTaxon <- NULL
    
    #Collapse any species values that are now synonymized
    working_wide <- aggregate(Count ~ Sample + Taxon, FUN = sum, data=working_wide)
    
    #Convert from long format back to wide format
    working_wide <- dcast(working_wide, Sample ~ Taxon)
    
    #Export working_wide to .csv
    # write.csv(working_wide, "OUTPUT_renamed_data.csv", row.names = FALSE)
    # #export changed name list
    # write.csv(changed_names, "OUTPUT_name_changes.csv", row.names = FALSE)
    
    output<-list(UpdatedDataset = working_wide, ChangedNames = changed_names, NameTranslationSheet=working_translation)
    return(output)
    
    #Return error message if data_type string is misspelled
  } else { return(print("ERROR: Please choose either 'List' or 'Dataframe' for data_type"))}
}


# This function imports a species list given a csv translation file and columns containing different name adjustments.
# The position of the column with the preferred species list is given as colnumber.
importSpeciesList <- function(csv, colnumber) {
  species_list <- read.csv(csv, na.strings=c("","NA"))
  species_list <- species_list[ , c(1,colnumber), drop = T]
  names(species_list) <- c("name_orig", "name_fixed")
  
  # Make third column of old names replaced by new names if relevant; drop middle column
  my.na <- is.na(species_list$name_fixed)
  species_list$name_harmonized[my.na] <- as.character(species_list$name_orig[my.na])
  species_list$name_harmonized[!my.na] <- as.character(species_list$name_fixed[!my.na])
  
  # Drop intermediate
  species_list <- species_list[ , c(1,3), drop = T]
  
  return(species_list)
}

# This function takes a wide dataset and a name translation sheet, outputs new wide dataset with collapsed columns, according to translation sheet.
nameTranslate <- function(wide_df, name_df) {
  wide_df[wide_df == 0] <- NA
  df_long <- melt(wide_df, id.vars="UID", variable.name = "TAXON", value.name = "COUNT")
  df_long <- df_long[complete.cases(df_long[,3]),]
  df_long$fixedTAXON <- lookup_e(df_long$TAXON, name_df)
  
  df_long <- df_long[,c("UID", "fixedTAXON", "COUNT")]
  wide_df_adj <- dcast(df_long, UID ~ fixedTAXON, sum)
  
  #convert UID to character string
  wide_df_adj$UID <- as.character(wide_df_adj$UID)
  #order df, UID first, then alphabetical
  wide_df_adj <- wide_df_adj[,c(names(wide_df_adj)[1],sort(names(wide_df_adj)[-1]))] 
  
  return(wide_df_adj)
}


### Random forest model for detecting OTUs with high analyst signal
# Required installed packages: foreach, doParallel, iterators, parallel, randomForest
# Function rfmod requires dataframes: ss, wq, siteinfo, latlong, and taxonomist
# ss is a species (columns) by samples (rows) matrix
# wq is water quality data
# siteinfo is other environmental variables including physical habitat, ecoregion, etc.
# latlong is latitude and longitude of sites
# taxonomist is analyst identity of samples as binary factors
# All required dataframes have a common column of site IDs called "UID"
# Function returns a dataframe of the top 5 predictors of relative abundance of OTUs, 
# which are ordered by highest analyst signal (mse.taxid.sc) and number of occurrence.

rfmod <- function(ss, wq, siteinfo, latlong, taxonomist, seedvalue, minnumocc=50) {
  
  # get number of occurrences (total valves counted per sample)
  tnames <- names(ss)[-c(1,2)]
  numocc <- apply(ss[, tnames], 2, function(x) sum(x>0))
  tabund <- apply(ss[, tnames], 1, sum)
  
  for (i in tnames) {
    ss[, i] <- ss[,i]/tabund
  }
  
  # print number of sites with less than 400 valves counted
  print(sum(tabund < 400))
  
  # drop sites with less than 400 valves counted
  ss <- ss[tabund > 400,]
  
  # select number of OTUs to model.
  # print OTUs ordered by number of occurrences
  print(sort(numocc))
  
  # print number of OTUs selected
  print(table(numocc))
  incvec <- numocc > minnumocc  # select the minimum number of occurrences for an OTU (50 is default)
  cat("Number of OTUs selected:", sum(incvec), "\n")
  
  #    tnames.sel <- names(rev(sort(numocc)))[1:50]
  tnames.sel <- names(numocc)[incvec]
  
  # model most common OTU first
  # print number of sites
  print(nrow(ss))
  ss <- ss[, c("UID", tnames.sel)]
  
  # merge in water quality info
  ss <- merge(ss, wq, by = "UID")
  pred <- names(wq)[-1]
  
  # merge in siteinfo
  ss <- merge(ss, siteinfo, by = "UID")
  
  # select siteinfo variables to include as predictor variables in the model
  pred <- c(pred, "XCMGW", 
            "W1_HAG", "W1_HNOAG",
            "PCT_SAFN", 
            "LDCBF_G08", "LRBS_G08",
            "LOWFLOW", "BANKFULLFLOW", "SHADE", "MAVFLOWV",
            "WSAREA_NARS", 
            "eco9CPL", "eco9NAP", "eco9NPL", "eco9SAP", "eco9SPL", 
            "eco9TPL", "eco9UMW", "eco9WMT", "eco9XER",
            "rtR", "rtS", "rtT")
  
  # merge in taxonomist
  names0 <- names(taxonomist)
  taxnames <- names0[regexpr("tax", names0) != -1]
  print(taxnames)
  
  # convert taxonomist indicator variable to level
  taxonomist$taxid <- rep(NA, times = nrow(taxonomist))
  for (i in taxnames) {
    incvec <- taxonomist[,i] == 1
    taxonomist$taxid[incvec] <- i
  }
  taxonomist$taxid <- factor(taxonomist$taxid)
  ss <- merge(ss, taxonomist[, c("UID", "taxid", taxnames)], by = "UID")
  
  pred <- c(pred, "taxid")
  
  # merge in site location
  ss <- merge(ss, latlong[, c("UID", "LAT_DD83", "LON_DD83")],
              by = "UID")
  pred <- c(pred, "LAT_DD83", "LON_DD83")
  itaxa <- which(pred == "taxid")
  
  # print predictor variables included in the model
  print(pred)
  
  # function to run RF on different cores
  runchunk <- function(tnames.sel, ss, pred, itaxa) {
    require(randomForest)
    mse <- data.frame(matrix(NA, ncol = 8, nrow = length(tnames.sel)))
    mse[,1] <- tnames.sel
    # run RF
    for (i in 1:length(tnames.sel)) {
      set.seed(seedvalue)
      mod <- randomForest(ss[, pred], y = ss[,tnames.sel[i]], importance = T)
      # save the mse of taxa id and of the highest mse
      mse[i,2] <- max(mod$importance[,1])
      mse[i,3] <- mod$importance[itaxa, 1]
      var5 <- dimnames(mod$importance)[[1]][rev(order(mod$importance[,1]))][1:5]
      print(var5)
      mse[i,4:8] <- var5
      print(mse)
      print(mod$importance)
    }
    return(mse)
  }
  
  inparallel <- TRUE
  
  if (inparallel) {
    # split tnames to each processor
    nnames <- length(tnames.sel)
    ncore <- 7  # select number of cores here
    percore <- nnames/ncore
    istart <- round((0:(ncore-1))*percore)+1
    iend <- round((1:ncore)*percore)
    #        print(istart)
    #        print(iend)
    
    require(foreach)
    require(doParallel)
    cl <- makeCluster(ncore)
    registerDoParallel(cl)
    dfmse <- foreach(i=1:ncore, .combine = rbind) %dopar%
      runchunk(tnames.sel[istart[i]:iend[i]], ss, pred, itaxa)
    stopCluster(cl)
  }
  else {
    dfmse <- runchunk(tnames.sel[1:2], ss, pred, itaxa)
  }
  
  names(dfmse) <- c("tname", "mse.max", "mse.taxid", "pred1", "pred2",
                    "pred3",
                    "pred4", "pred5")
  
  
  dfmse$mse.taxid.sc <- dfmse$mse.taxid/dfmse$mse.max
  dfmse$numocc <- numocc[tnames.sel]
  dfmse_ordered <- dfmse[order(-dfmse$mse.taxid.sc, -dfmse$numocc),]
  
  print(head(dfmse_ordered, n=20))
  return(dfmse_ordered)
}


#Function for processing species data
#Rare species removed if less than 50 occurrences
#Samples removed if less than 400 valves counted
ssProcess <- function (ss, taxonomist) {
  
  # get number of occurrences
  tnames <- names(ss)[-c(1,2)]
  numocc <- apply(ss[, tnames], 2, function(x) sum(x>0))
  tabund <- apply(ss[, tnames], 1, sum)
  
  for (i in tnames) {
    ss[, i] <- ss[,i]/tabund
  }
  # print(sum(tabund < 400))
  
  # drop sites with less than 400 individuals
  ss <- ss[tabund > 400,]
  
  print(table(numocc))
  incvec <- numocc > 50  # select the minimum number of occurrences for a taxa
  cat("Number of taxa selected:", sum(incvec), "\n")
  
  #    tnames.sel <- names(rev(sort(numocc)))[1:50]
  tnames.sel <- names(numocc)[incvec]
  
  # model most common taxa first
  print(nrow(ss))
  ss <- ss[, c("UID", tnames.sel)]
  
  # merge in taxonomist
  names0 <- names(taxonomist)
  taxnames <- names0[regexpr("tax", names0) != -1]
  print(taxnames)
  
  # convert taxonomist indicator variable to level
  taxonomist$taxid <- rep(NA, times = nrow(taxonomist))
  for (i in taxnames) {
    incvec <- taxonomist[,i] == 1
    taxonomist$taxid[incvec] <- i
  }
  taxonomist$taxid <- factor(taxonomist$taxid)
  ss <- merge(taxonomist[, c("UID", "taxid")], ss, by = "UID")
  
  return(ss)
}

#Function for plotting ellipsoid hulls around groups from an nmds ordination object
#Prints the goodness of fit information for the analyst groups based on group centroids
plotEhulls <- function(mdscomm, taxdata) {
  plot(mdscomm, display = "sites", type = "n", xlim=c(-2, 2), ylim=c(-2,2.5))
  stats <- with(mdscomm,
                ordiellipse(mdscomm, taxdata$taxid, kind="ehull", conf=0.95, 
                            lwd=4, draw = "polygon", col=NULL, show.groups="taxEM",
                            border = "green"))
  with(mdscomm,
       ordiellipse(mdscomm, taxdata$taxid, kind="ehull", conf=0.95, 
                   lwd=4, draw = "polygon", col=NULL, show.groups="taxGL",
                   border = "grey"))
  with(mdscomm,
       ordiellipse(mdscomm, taxdata$taxid, kind="ehull", conf=0.95, 
                   lwd=4, draw = "polygon", col=NULL, show.groups="taxJG",
                   border = "red"))
  with(mdscomm,
       ordiellipse(mdscomm, taxdata$taxid, kind="ehull", conf=0.95, 
                   lwd=4, draw = "polygon", col=NULL, show.groups="taxJS",
                   border = "skyblue"))
  with(mdscomm,
       ordiellipse(mdscomm, taxdata$taxid, kind="ehull", conf=0.95, 
                   lwd=4, draw = "polygon", col=NULL, show.groups="taxJZ",
                   border = "pink"))
  with(mdscomm,
       ordiellipse(mdscomm, taxdata$taxid, kind="ehull", conf=0.95, 
                   lwd=4, draw = "polygon", col=NULL, show.groups="taxKM",
                   border = "purple"))
  with(mdscomm,
       ordiellipse(mdscomm, taxdata$taxid, kind="ehull", conf=0.95, 
                   lwd=4, draw = "polygon", col=NULL, show.groups="taxLR",
                   border = "yellow"))
  with(mdscomm,
       ordiellipse(mdscomm, taxdata$taxid, kind="ehull", conf=0.95, 
                   lwd=4, draw = "polygon", col=NULL, show.groups="taxME",
                   border = "blue"))
  with(mdscomm,
       ordiellipse(mdscomm, taxdata$taxid, kind="ehull", conf=0.95, 
                   lwd=4, draw = "polygon", col=NULL, show.groups="taxSM",
                   border = "orange"))
  scrs <- scores(mdscomm, display = "sites")
  
  #envfit analyst to obtain group centroids
  fit <- envfit(mdscomm ~ taxdata$taxid)
  
  #plot the envfit centroid points
  points(fit$factors$centroids, pch = 16, col = "black", cex = 1)
  
  
  #return goodness of fit
  return(fit)
}

#Function for plotting results of nmds ordination object with analyst groups in different colors
plotPoints <- function (mdscomm, taxdata) {
  
  colvec <- c("green", "grey", "red", "skyblue", "pink", "purple", "yellow", "blue", "orange")
  
  plot(mdscomm, display = "sites", type = "n", xlim=c(-2, 2), ylim=c(-2,2))
  
  with(taxdata, points(mdscomm, display = "sites", col = colvec[taxdata$taxid],
                       pch = 21, bg = colvec[taxdata$taxid]))
}

#Modified function specresponse from goeveg package to include more plotting parameters
specresponse_sl <- function(species, var, main, xlab, xlim, ylim, model = "auto", method="env", axis=1, points = FALSE, bw = FALSE) {
  
  if(!is.data.frame(species)) {
    
    if(missing(main)) {
      main <- deparse(substitute(species))
    }
    
    specnames <- deparse(substitute(species))
    species <- data.frame(species)
    ismat <- F
    
  } else {
    if(missing(main)) {
      main <- ""
    }
    specnames <- names(species)
    ismat <- T
  }
  
  species <- decostand(species, method = "pa")
  
  if(length(species) >= 1) {
    
    ls <- length(species)
    
    if(method == "env") {
      
      if(missing(xlab)) {
        xlab <- deparse(substitute(var))
      }
    } else if(method == "ord") {
      
      # Extract site scores from ordination
      var <- as.numeric(scores(var, display="sites", choices=axis))
      
      # X-axis labeling
      if(missing(xlab)) {
        xlab <- paste("Axis", axis, "sample scores")
      }
    } else {
      stop("Method unknown.")
    }
    
    plot(var, species[,1], main = main, type="n",
         xlab = xlab, ylab="Probability of occurence", xlim=xlim, ylim = ylim)
    
    for(i in 1:ls) {
      
      if(length(species[species[,i]>0,i]) <= 5) {
        # tried warning instead of print
        warning(paste("Only", length(species[species[,i]>0,i]), "occurences of", names(species)[i], "."))
      }
      
      if(model == "unimodal") {
        
        specresponse <- suppressWarnings(glm(species[,i] ~ poly(var, 2),
                                             family="binomial"))
        
      } else if (model == "linear") {
        
        specresponse <- suppressWarnings(glm(species[,i] ~ var,
                                             family="binomial"))
        
      } else if (model == "bimodal") {
        
        specresponse <- suppressWarnings(glm(species[,i] ~ poly(var, 4),
                                             family="binomial"))
        
      }
      else if (model == "auto") {
        
        glm.1 <- suppressWarnings(glm(species[,i] ~ poly(var, 1), family="binomial"))
        glm.2 <- suppressWarnings(glm(species[,i] ~ poly(var, 2), family="binomial"))
        glm.3 <- suppressWarnings(glm(species[,i] ~ poly(var, 3), family="binomial"))
        glm.AIC <- c(extractAIC (glm.1)[2], extractAIC (glm.2)[2],
                     extractAIC (glm.3)[2])
        
        switch(c(1,2,3)[glm.AIC==min(glm.AIC)],
               {specresponse <- glm.1; deg<-1},
               {specresponse <- glm.2; deg<-2},
               {specresponse <- glm.3; deg<-3})
        
        print(paste("GLM with", deg, "degrees fitted for", specnames[i]))
        
      } else if (model == "gam") {
        
        gam.list <- list()
        gam.AIC <- 0
        
        for(n in 1:4) {
          gam.list[[paste("gam", n, sep=".")]] <- mgcv::gam(species[,i] ~ s(var, k = n+2), family='binomial')
          gam.AIC[n] <- extractAIC(gam.list[[n]])[2]
        }
        
        switch(c(1,2,3,4)[gam.AIC==min(gam.AIC)],
               {specresponse <- gam.list[[1]]; deg<-3},
               {specresponse <- gam.list[[2]]; deg<-4},
               {specresponse <- gam.list[[3]]; deg<-5},
               {specresponse <- gam.list[[4]]; deg<-6})
        
        print(paste("GAM with", deg, "knots fitted for", specnames[i]))
        
        
      } else {
        stop("Model unknown.")
      }
      
      xneu <- seq(min(var), max(var), len = 101)
      preds <- predict(specresponse, newdata = data.frame(var = xneu),
                       type="response")
      
      if(bw == T) {
        if(points == TRUE) {
          if(i > 1) {
            species[,i][species[,i]==1] <- species[,i][species[,i]==1] - 0.02*(i-1)
            species[,i][species[,i]==0] <- species[,i][species[,i]==0] + 0.02*(i-1)
          }
          
          col <- col2rgb("black")
          points(var, species[,i], pch = i,
                 col = rgb(col[1], col[2], col[3], maxColorValue = 255, alpha = 50))
        }
        
        lines(preds ~ xneu, lty=i)
      } else {
        
        if(points == TRUE) {
          if(i > 1) {
            species[,i][species[,i]==1] <- species[,i][species[,i]==1] - 0.02*(i-1)
            species[,i][species[,i]==0] <- species[,i][species[,i]==0] + 0.02*(i-1)
          }
          
          col <- col2rgb(i)
          points(var, species[,i],
                 col = rgb(col[1], col[2], col[3], maxColorValue = 255, alpha = 50),
                 pch = 16)
        }
        
        lines(preds ~ xneu, col = i)
      }
    }
    
    if(ismat == T) {
      
      if(bw == T){
        if(points == TRUE) {
          legend("top", xpd=TRUE, inset=-0.7, horiz=F, legend=names(species), lty=1:ls, pch=1:ls,
                 bty = "n", cex = 0.85)
        } else {
          legend("top", xpd=TRUE, inset=-0.7, horiz=F, legend=names(species), lty=1:ls,
                 bty = "n", cex = 0.85)
        }
      } else {
        legend("top", xpd=TRUE, inset=-0.7, horiz=F, legend=names(species), col=1:ls, lty=1,
               bty = "n", cex = 0.85)
      }
    }
    
  }  else {
    stop("No species in matrix.")
  }
}

